%wrapper.m
%For questions about this file, please contact Stephen Terry at
%sterry@stanford.edu
%
%Replication code to estimate conditional heteroskedasticity of quarterly US TFP growth
%and compute moments of the implied series. Also computes RBC moments.

%Used for the project "Really Uncertain Business Cycles"
%Nicholas Bloom, Max Floetotto, Nir Jaimovoich, Itay Saporta-Eksten, and Stephen Terry

%This version of the code: May 18, 2012, Stephen Terry

%Data sources used in creation of GARCH(1,1) graph in the paper:
%quarterly_tfp.xls - John Fernald's TFP series (downloaded on May 17, 2012)
%recessions.xls - NBER quarterly recession dates from FRED (downloaded on May 17, 2012)
%Estimation sample is 1972-2010.

%Data sources used in creation of RBC moment table in paper:
%bcmoments_dta.xls - FRED file with quarterly data for GDPC96, GPDIC96,
%PCECC96,HOANBS, sample used is 1972-2010.

%Input files:
%dtfp_input.xls - raw "dtfp" series from Fernald's data
%bcmoments_input.xls - raw data from FRED

%This files requires Ellen R. McGrattan's hptrend.m to run

close all; clear all; clc;

%%%%%%%%%%%%%%GARCH(1,1) estimation

dTFP_data=xlsread('dtfp_input.xls');
dTFP=dTFP_data(:,1);
dTFP=double(dTFP);
spec=garchset('P',1,'Q',1);
[Coeff,Errors,LLF,Innovations,Sigmas,Summary]=garchfit(spec,dTFP-mean(dTFP));
TFP_stats(1)=mean(Sigmas);
TFP_stats(2)=std(Sigmas);
TFP_stats(3)=skewness(Sigmas);
TFP_stats(4)=corr(Sigmas(2:end),Sigmas(1:end-1));

disp('**********************************************************');
disp('**********************************************************');
disp('Differenced TFP fitted GARCH(1,1) conditional heteroskedasticity - AVG, STD DEV, SKEWNESS, 1ST AUTOCORR')
display(num2str(TFP_stats([1 2 3 4])));
disp('**********************************************************');
disp('**********************************************************');
disp('  ')
display('*****estimated heteroskedasticity series:')
display(num2str(Sigmas))
display('*****')
disp(' ')


%regression of estimated htrsky on NBER recession dates
recession = dTFP_data(:,2);
whichstats={'beta','rsquare','tstat'};
stats=regstats(log(Sigmas),recession,'linear',whichstats);
tstats=stats.tstat.t;
betas=stats.beta;
disp('**********************************************************');
disp('**********************************************************');
disp('Estimated GARCH(1,1) conditional heteroskedasticity series regressed on recession dates:')
disp('log(Sigma_t) = beta_0 + beta_1 * recession_t + e_t')
disp(['beta_0 = ' num2str(betas(1))])
disp(['beta_1 = ' num2str(betas(2))])
disp(['t-stat for beta_1 = ' num2str(tstats(2))])
disp('**********************************************************');
disp('**********************************************************');
disp('  ')

%%%%%%%%%%%%%%%RBC moment calculation

%read in data from FRED, 1972-2010
[num,txt,raw]=xlsread('bcmoments_input.xls');
y=num;
logy=log(y);
[logyt,logyd]=hptrend(logy,1600);
logyt=100*logyt;
logyd=100*logyd;

%now, compute BC volatilities, relative volatilities
stdvec=std(logyd,1,1)';
stdvec_norm=stdvec/stdvec(1);

%now, compute autocorrelations
corrvec=zeros(4,1);
for serct=1:4;
    dum=corr([logyd(:,serct) logyd(:,1)]);
    corrvec(serct)=dum(1,2);
end;

outmat = [stdvec stdvec_norm corrvec];

%print output
disp('  ')
disp('******************************')
disp('******business cycle moments')
disp('series, sigma(x),  sigma(x)/sigma(y),  rho(x,y)')
for serct=1:4
    disp([txt{serct} '       ' num2str(outmat(serct,:))])
end;
disp('*****')